#! /bin/bash
### Linux 64 bit #######################
# Ubuntu 18.04.x ​​(LTS) is recommended  #
# Packages required:                   #
#       dialog gcc make zip            #
###########################################
# Configure where we can find things here #
NDK=r21d
###############
#    true|false
API_MENU=false
ARCH_MENU=false
#########################
# Android.mk # "xxx".mk #
MK=false
##########
dir=$(pwd)
#######################
# Android.mk checking #
for i in openssl libusb ; do [ ! -e "$dir/${i}.mk" ] && MK=false; done 
####################################
# Checking the number of CPU cores #
cores=$(cat /proc/cpuinfo | grep "cpu cores" | awk -F: '{ num+=$2 } END{ print num }')
#########################
# Download Android NDK. #
[ ! -d $dir/android-ndk-$NDK ] && wget -c -P $dir --progress=bar:force "https://dl.google.com/android/repository/android-ndk-$NDK-linux-x86_64.zip" 2>&1 | while read -d "%" X; do sed 's:^.*[^0-9]\([0-9]*\)$:\1:' <<<"$X"; done | dialog --title "" --clear --stdout --gauge "Download: android-ndk-$NDK-linux-x86_64.zip" 6 50
##################
# Unzip the NDK. #
[ ! -d $dir/android-ndk-$NDK ] && unzip -d $dir $dir/android-ndk-$NDK-linux-x86_64.zip
############
# API Menu #
API="16"
###
if $API_MENU; then
	clear
	_16="4.1_Jelly_Bean"
	_17="4.2_Jelly_Bean"
	_18="4.3_Jelly_Bean"
	_19="4.4_KitKat"
	_20="Wear_4.4_KitKat"
	_21="5.0_Lollipop"
	_22="5.1_Lollipop"
	_23="6.0_Marshmallow"
	_24="7.0_Nougat"
	_25="7.1_Nougat"
	_26="8.0_Oreo"
	_27="8.1_Oreo"
	_28="9.0_Pie"
	_29="10.0_Q"
	list=
	for i in $(seq 16 29); do
		p="_$i"
		[ -d $dir/android-ndk-$NDK/platforms/android-$i ] && list="$list $i ${!p} "
	done
	list=($list)
	selected=$(dialog --stdout --clear --colors --backtitle $0 --menu "Android NDK $NDK" 16 40 10 ${list[@]})
	case $selected in
	1* | 2*) API=$selected ;;
	*) exit ;;
	esac
fi
#############
# ARCH Menu #
ARCH="arm"
###
if $ARCH_MENU; then
	clear
	arch="arm arm64 x86 x86_64"
	s="12 40 4"
	[ "$API" -lt "21" ] && arch="arm x86" && s="10 40 2"
	p="_$API"
	list=
	for i in $arch; do list="$list $i "android$API" "; done
	list=($list)
	selected=$(dialog --stdout --clear --colors --backtitle $0 --menu "${!p}" $s ${list[@]})
	case $selected in
	arm | arm64 | x86 | x86_64) ARCH=$selected ;;
	*) exit ;;
	esac
fi
################
FLAGS="-fpic -fno-addrsig -ffunction-sections -fdata-sections -funwind-tables -fstack-protector-strong -no-canonical-prefixes -Wno-invalid-command-line-argument -Wno-unused-command-line-argument -DANDROID -D__ANDROID_API__=$API"
case "$ARCH" in
arm)
	ABI="armeabi-v7a"
	HOST="arm-linux-androideabi"
	TARGET="armv7a-linux-androideabi$API"
	FLAGS="-march=armv7-a -mtune=cortex-a9 -mfloat-abi=softfp -mfpu=vfpv3-d16 $FLAGS"
	;;
x86)
	ABI="$ARCH"
	HOST="i686-linux-android"
	TARGET="${HOST}${API}"
	;;
arm64)
	ABI="arm64-v8a"
	HOST="aarch64-linux-android"
	[ "$API" -lt "21" ] && API="21"
	TARGET="${HOST}${API}"
	;;
x86_64)
	ABI="$ARCH"
	HOST="x86_64-linux-android"
	[ "$API" -lt "21" ] && API="21"
	TARGET="${HOST}${API}"
	;;
esac
#######################
# Export Android NDK. #
export ANDROID_NDK=$dir/android-ndk-$NDK
export PATH=$ANDROID_NDK/toolchains/llvm/prebuilt/linux-x86_64/bin:$PATH
export CC="clang --target=$TARGET $FLAGS -I$prefix/include -L$prefix/lib"
export CXX="clang++ --target=$TARGET $FLAGS -I$prefix/include -L$prefix/lib"
##############################
# Create a prefix directory. #
#prefix="$ANDROID_NDK/sysroot/$ARCH-$API"
prefix="$ANDROID_NDK/sysroot"
[ ! -d $prefix/lib ] && mkdir -p $prefix/lib
[ ! -d $prefix/include ] && mkdir -p $prefix/include
#################################
# Download and Install OpenSSl. #
if [ ! -e $prefix/lib/libcrypto.a ]; then
	build=openssl-1.1.1g
	[ ! -e $dir/$build.tar.gz ] && wget -c -P $dir https://www.openssl.org/source/$build.tar.gz
	[ ! -d $dir/$build ] && tar -C $dir -xf $dir/$build.tar.gz
	cd $dir/$build
	./Configure android-$ARCH --api=0.9.8 -D__ANDROID_API__=$API no-afalgeng no-aria no-asan no-asm no-async no-autoalginit no-autoerrinit no-autoload-config no-bf no-blake2 no-camellia no-capieng no-cast no-chacha no-cmac no-cms no-comp no-crypto-mdebug no-crypto-mdebug-backtrace no-ct no-devcryptoeng no-dgram no-dh no-dsa no-dso no-dtls no-dynamic-engine no-ec no-ec2m no-ecdh no-ecdsa no-ec_nistp_64_gcc_128 no-egd no-engine no-err no-external-tests no-filenames no-fuzz-libfuzzer no-fuzz-afl no-gost no-heartbeats no-idea no-makedepend no-md2 no-mdc2 no-md4 no-msan no-multiblock no-nextprotoneg no-ocb no-ocsp no-pic no-poly1305 no-posix-io no-psk no-rc2 no-rc4 no-rc5 no-rdrand no-rfc3779 no-rmd160 no-scrypt no-sctp no-seed no-shared no-siphash no-sm2 no-sm3 no-sm4 no-srp no-srtp no-sse2 no-ssl no-ssl-trace no-tests no-threads no-tls no-ts no-ubsan no-ui-console no-unit-test no-whirlpool no-weak-ssl-ciphers no-zlib no-zlib-dynamic no-ssl3 no-ssl3-method no-tls1 no-tls1-method no-tls1_1 no-tls1_1-method no-tls1_2 no-tls1_2-method no-tls1_3 no-dtls1 no-dtls1-method no-dtls1_2 no-dtls1_2-method --prefix=$prefix
	if ! $MK; then
		make -C $dir/$build include/crypto/bn_conf.h >/dev/null
		make -C $dir/$build include/crypto/dso_conf.h >/dev/null
		make -C $dir/$build crypto/buildinf.h >/dev/null
		make -C $dir/$build include/openssl/opensslconf.h >/dev/null
		echo "OPENSSL := $dir/$build" >>$dir/config.mk
		cd $dir
		cp $dir/openssl.mk $dir/$build/Android.mk
		$ANDROID_NDK/ndk-build APP_ABI=$ABI APP_PLATFORM=android-$API NDK_PROJECT_PATH=$dir APP_BUILD_SCRIPT=$dir/$build/Android.mk
		mv $dir/obj/local/$ABI/libcrypto.a $prefix/lib
		mv $dir/obj/local/$ABI/libssl.a $prefix/lib
		[ ! -d $prefix/include/openssl ] && cp -r $dir/$build/include/openssl $prefix/include/
	else
		make clean
		make -j$cores install_sw
	fi
	rm -rf $dir/$build $dir/*obj* $dir/config.mk
	clear
	[ ! -e $prefix/lib/libcrypto.a ] && echo "ERROR! $build" && exit
	cd $dir
fi
################################
# Download and Install libusb. #
if [ ! -e $prefix/lib/libusb-1.0.a ]; then
	build=libusb-1.0.23
	[ ! -e $dir/$build.tar.bz2 ] && wget -c -P $dir https://github.com/libusb/libusb/releases/download/v1.0.23/$build.tar.bz2
	[ ! -d $dir/$build ] && tar -C $dir -jxf $dir/$build.tar.bz2
	if ! $MK; then
		echo "LIBUSB := $dir/$build" >>$dir/config.mk
		cd $dir
		cp $dir/libusb.mk $dir/$build/Android.mk
		$ANDROID_NDK/ndk-build APP_ABI=$ABI APP_PLATFORM=android-$API NDK_PROJECT_PATH=$dir APP_BUILD_SCRIPT=$dir/$build/Android.mk
		mv $dir/obj/local/$ABI/libusb-1.0.a $prefix/lib
		[ ! -d $prefix/include/libusb-1.0/libusb.h ] && mkdir -p $prefix/include/libusb-1.0 && cp $dir/$build/libusb/libusb.h $prefix/include/libusb-1.0/
	else
		cd $dir/$build
		./configure --host=$HOST --enable-static --disable-shared --disable-udev --prefix=$prefix
		cp $dir/$build/android/config.h $dir/$build/config.h
		make clean
		make -j$cores install
	fi
	rm -rf $dir/$build $dir/*obj* $dir/config.mk
	clear
	[ ! -e $prefix/lib/libusb-1.0.a ] && echo "ERROR! $build" && exit
	cd $dir
fi
#################################
# Download and Install libcurl. #
if ! $MK && [ ! -e $prefix/lib/libcurl.a ]; then
	export CFLAGS="$FLAGS -I$prefix/include -L$prefix/lib"
	export CPPFLAGS="$FLAGS -DCURL_STATICLIB -I$prefix/include"
	export LDFLAGS="$FLAGS -L$prefix/lib"
	build=curl-7.78.0
	[ ! -e $dir/$build.tar.xz ] && wget -c -P $dir http://curl.haxx.se/download/$build.tar.xz
	[ ! -d $dir/$build ] && tar -C $dir -xf $dir/$build.tar.xz
	cd $dir/$build
	./configure --host=$HOST \
		--target=$HOST \
		--prefix=$prefix \
		--disable-shared \
		--enable-static \
		--disable-debug \
		--enable-optimize \
		--enable-warnings \
		--disable-curldebug \
		--disable-ares \
		--disable-largefile \
		--enable-http \
		--disable-ftp \
		--disable-file \
		--disable-ldap \
		--disable-ldaps \
		--disable-rtsp \
		--enable-proxy \
		--disable-dict \
		--disable-telnet \
		--disable-tftp \
		--disable-pop3 \
		--disable-imap \
		--disable-smb \
		--disable-smtp \
		--disable-gopher \
		--disable-mqtt \
		--disable-manual \
		--enable-libgcc \
		--enable-ipv6 \
		--disable-versioned-symbols \
		--disable-nonblocking \
		--disable-threaded-resolver \
		--disable-verbose \
		--disable-sspi \
		--disable-crypto-auth \
		--enable-cookies \
		--disable-symbol-hiding \
		--disable-soname-bump \
		--with-gnu-ld \
		--without-krb4 \
		--without-spnego \
		--without-gssapi \
		--without-zlib \
		--without-egd-socket \
		--disable-thread \
		--with-random=/dev/urandom \
		--without-gnutls \
		--without-pic \
		--with-ca-path=/system/etc/security/cacerts \
		--without-polarssl \
		--without-nss \
		--without-ca-path \
		--without-libpsl \
		--without-libssh2 \
		--without-libidn \
		--without-libidn2 \
		--without-nghttp2

	make clean
	make -j$cores install
	rm -rf $dir/$build #$dir/*obj* $dir/config.mk
	clear
	[ ! -e $prefix/lib/libcurl.a ] && echo "ERROR! $build" && exit
	cd $dir
fi
#################################
# Download and Install libpng. #
if ! $MK && [ ! -e $prefix/lib/libpng.a ]; then
	export CFLAGS="$FLAGS -I$prefix/include -L$prefix/lib"
	export CPPFLAGS="$FLAGS -DCURL_STATICLIB -I$prefix/include"
	export LDFLAGS="$FLAGS -L$prefix/lib"
	build=libpng-1.6.37
	[ ! -e $dir/$build.tar.gz ] && wget -c -P $dir https://deac-ams.dl.sourceforge.net/project/libpng/libpng16/1.6.37/$build.tar.gz
	[ ! -d $dir/$build ] && tar -C $dir -zxf $dir/$build.tar.gz
	cd $dir/$build
	./configure --host=$HOST --disable-shared --enable-static --prefix=$prefix
	make clean
	make -j$cores install
	rm -rf $dir/$build #$dir/*obj* $dir/config.mk
	#clear
	[ ! -e $prefix/lib/libpng.a ] && echo "ERROR! $build" && exit
	cd $dir
fi
######
exit
